# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


"""
Usage:
   netq show opta-health [json]

Options:
   opta-health                  : Display health of apps on the OPTA
"""
from __future__ import absolute_import
from __future__ import print_function

import ujson as json
import requests
from collections import OrderedDict
from datetime import datetime
from tabulate import tabulate
from netq_lib.common.utils import RED_T, COLOR_GREEN, get_logger

from netq_lib.common.configuration import (
    get_config,
)
from netq_lib.common.enums import (
    ConfKw,
    RunningService,
    DefaultPaths
)

from netq_apps.modules import (
    NetqModule,
    RC_SUCCESS,
)
app = NetqModule()


@app.route('show opta-health')
def config_show_opta_health(cli, netq):
    current_config = get_config(RunningService.NETQD)
    logger = get_logger(ConfKw.NETQD, current_config)
    netq_cli = current_config.get(ConfKw.NETQ_CLI)
    if not netq_cli.get(ConfKw.SERVER):
        error_msg = 'server is not defined under netq-cli in {}\n' \
                    'Please add netq-cli server ' \
                    '(netq config add cli server <text-gateway-ip>)'.format(DefaultPaths.CONF_FILE)
        logger.error(error_msg)
        raise Exception(error_msg)

    url = 'http://{}:{}/netq/health/v1/healthsummary'.format(netq_cli.get(ConfKw.SERVER),
                                                             netq_cli.get(ConfKw.HEALTH_PORT))
    try:
        logger.info('GET request sent {}'.format(url))
        response = requests.get(url=url, timeout=180)
        if response.status_code != requests.codes.ok:
            raise requests.exceptions.HTTPError(response.json())
        response_json = response.json()
        if cli.get('json'):
            print(json.dumps(response_json, sort_keys=True, indent=4))
        else:
            json_keys = OrderedDict([('agentName', 'Application'), ('healthStatus', 'Status'), ('health', 'Health'),
                                     ('kafka_stream_status', 'Kafka Stream'), ('git_hash', 'Git Hash'),
                                     ('time', 'Timestamp')])
            headers = json_keys.values()
            table = []
            for entry in response_json:
                health_data = entry.get('healthResultData', dict())
                health_status = health_data.get('healthStatus', '')
                row = [entry.get('agentName', ''),
                       COLOR_GREEN(health_status) if health_status == 'UP' else RED_T(health_status),
                       '', '', '', datetime.fromtimestamp(entry.get('time', 0) / 1000).ctime()]
                meta_data = health_data.get('metaData', list())
                for data in meta_data:
                    row[json_keys.keys().index(data.get('name'))] = data.get('value')
                table.append(row)
            print(tabulate(table, headers=headers, tablefmt='simple'))
            print()
    except Exception as ex:
        logger.error(ex)
        raise requests.exceptions.HTTPError('GET request failed {}'.format(url))
    else:
        return RC_SUCCESS
